package model;

import java.util.ArrayList;

public class Bishop extends Piece {

    public Bishop(Position p, int color) {
        super(p, color);
    }

    public ArrayList<Position> getAvailableMoves(Board b) {

        // creates a dynamic list of available moves
        ArrayList<Position> moves = new ArrayList<>();
        Position arrival;

        // list of the 4 bishop directions
        int[] all_x = {1, 1, -1, -1};
        int[] all_y = {1, -1, -1, 1};

        for(int n = 0 ; n < all_x.length ; n++) { // for each direction
            int x = all_x[n];
            int y = all_y[n];
            boolean blocked = false;
            int i = 1;
            while (i < 8 && !blocked) { // continues "as a ray" in that direction until blocked
                arrival = new Position(p.getX() + (x * i), p.getY() + (y * i));
                if (b.isEnemy(arrival, p) || b.isFree(arrival)) {
                    moves.add(arrival);
                }
                if (!b.isFree(arrival)) {
                    blocked = true;
                }
                i++;
            }
        }

        // Full wrote version : --------------------------------------------
        /*
        Position Cursor = new Position(p.getX(), p.getY());
        int i = 1;
        boolean leftTop = false;
        boolean rightTop = false;
        boolean leftDown = false;
        boolean rightDown = false;
        while (i < 8) {
            //left diagonal
            arrival = new Position(p.getX() + i, p.getY() + i);
            if (!leftTop) {
                if (b.isEnemy(arrival, p) || b.isFree(arrival)) {
                    moves.add(arrival);
                }
            }
            if (!b.isFree(arrival)) {
                leftTop = true;
            }
            //right diagonal
            arrival = new Position(p.getX() - i, p.getY() + i);
            if (!rightTop) {
                if (b.isEnemy(arrival, p) || b.isFree(arrival)) {
                    moves.add(arrival);
                }
            }
            if (!b.isFree(arrival)) {
                rightTop = true;
            }
            arrival = new Position(p.getX() + i, p.getY() - i);
            if (!leftDown) {
                if (b.isEnemy(arrival, p) || b.isFree(arrival)) {
                    moves.add(arrival);
                }
            }
            if (!b.isFree(arrival)) {
                leftDown = true;
            }
            arrival = new Position(p.getX() - i, p.getY() - i);
            if (!rightDown) {
                if (b.isEnemy(arrival, p) || b.isFree(arrival)) {
                    moves.add(arrival);
                }
            }
            if (!b.isFree(arrival)) {
                rightDown = true;
            }
            i++;
        }
        */
        // -----------------------------------------------------------

        return moves;
    }

    public Piece newInstanceCopy() {
        return new Bishop(p, color);
    }
}
