package view;

import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.collections.ObservableListBase;
import javafx.scene.control.ComboBox;
import javafx.scene.effect.Glow;
import javafx.scene.image.Image;
import javafx.scene.image.ImageView;
import javafx.scene.input.MouseEvent;
import javafx.scene.layout.AnchorPane;
import javafx.scene.paint.Color;
import javafx.scene.shape.Rectangle;

public class HUD {

    private Image standard_img, variant_img, rollback_img;
    private ImageView standard_imgView, variant_imgView, rollback_imgView;
    private Rectangle darkener;

    private ComboBox combo;
    private boolean comboShowed;

    public HUD() {
        standard_img = new Image("/standard.jpg");
        variant_img = new Image("/variant.jpg");
        standard_imgView = new ImageView(standard_img);
        variant_imgView = new ImageView(variant_img);
        standard_imgView.setPreserveRatio(true);
        variant_imgView.setPreserveRatio(true);
        darkener = new Rectangle();
        darkener.setX(0);
        darkener.setY(0);
        darkener.setFill(Color.BLACK);
        darkener.setOpacity(0.5);
        Glow glow = new Glow(0.2);
        Glow none = new Glow(0);

        standard_imgView.addEventHandler(MouseEvent.MOUSE_ENTERED, event -> {
            standard_imgView.setEffect(glow);
            variant_imgView.setEffect(none);
        });
        standard_imgView.addEventHandler(MouseEvent.MOUSE_EXITED, event -> {
            standard_imgView.setEffect(none);
        });
        variant_imgView.addEventHandler(MouseEvent.MOUSE_ENTERED, event -> {
            variant_imgView.setEffect(glow);
        });
        variant_imgView.addEventHandler(MouseEvent.MOUSE_EXITED, event -> {
            variant_imgView.setEffect(none);
        });

        rollback_img = new Image("/rollback.png");
        rollback_imgView = new ImageView(rollback_img);
        rollback_imgView.setPreserveRatio(true);
        rollback_imgView.setFitWidth(64);
        rollbackHide();

        String pieces[] = {"Queen", "Rook", "Bishop", "Knight"};
        combo = new ComboBox(FXCollections.observableArrayList(pieces));
        combo.getSelectionModel().select(0);
        combo.setStyle("-fx-font: 58 arial;");
        combo.setTranslateX(64);
        combo.setTranslateY(64);
        comboShowed = false;
    }

    public void resize(double width, double height) {
        int weight = 2;
        int total = weight*2 + 4;
        int avg = (int) (width + height) / 2;
        standard_imgView.setX(width/total);
        standard_imgView.setY(height/total/2);
        standard_imgView.setFitWidth(width*weight/total);
        variant_imgView.setX(width/total * (weight+3));
        variant_imgView.setY(height/total/2);
        variant_imgView.setFitWidth(width*weight/total);
        darkener.setWidth(width);
        darkener.setHeight(height);
        rollback_imgView.setX(width - 36 - (avg / 12));
        rollback_imgView.setY(height - 56 - (avg / 12));
        rollback_imgView.setFitWidth(avg / 12);
        //combo.setStyle("-fx-font: " + avg / 24 + " arial;");
        //combo.setTranslateX(avg / 24);
        //combo.setTranslateY(avg / 24);
        //combo.resize(16 * avg / 24, 16 * avg / 24);
    }

    public ImageView getStandard() {
        return standard_imgView;
    }

    public ImageView getVariant() {
        return variant_imgView;
    }

    public ImageView getRollback() {
        return rollback_imgView;
    }

    public void rollbackHide() {
        rollback_imgView.setOpacity(0);
    }
    public void rollbackShow() {
        rollback_imgView.setOpacity(1);
    }

    public void removeMenu(AnchorPane group2d) {
        group2d.getChildren().removeAll(darkener, standard_imgView, variant_imgView);
    }
    public void addAllToScene(AnchorPane group2d) {
        group2d.getChildren().addAll(darkener, standard_imgView, variant_imgView, rollback_imgView);
    }

    public void addCombo(AnchorPane group2d) {
        if(!comboShowed) {
            group2d.getChildren().add(combo);
            comboShowed = true;
        }
    }

    public void removeCombo(AnchorPane group2d) {
        if(comboShowed) {
            group2d.getChildren().remove(combo);
            comboShowed = false;
        }
    }

    public int getComboID() {
        String Valeur = (String) combo.getValue();
        int id;
        switch(Valeur){
            case "Queen" -> id = 0;
            case "Rook" -> id = 1;
            case "Bishop" -> id = 2;
            default -> id = 3;
        }
        return id;
    }
}
